@@ -38,6 +38,7 @@
 #undef isnan
 #endif
 
+/*OUTER_NAMESPACE_BEGIN*/
 namespace Vc
 {
 namespace AVX
@@ -226,13 +227,13 @@ template<typename T> class Vector
         //postfix
         Vc_ALWAYS_INLINE Vector operator++(int) { const Vector<T> r = *this; data() = VectorHelper<T>::add(data(), VectorHelper<T>::one()); return r; }
 
-        Vc_INTRINSIC Common::AliasingEntryHelper<StorageType> operator[](int index) {
+        Vc_INTRINSIC Common::AliasingEntryHelper<StorageType> operator[](size_t index) {
 #if defined(VC_GCC) && VC_GCC >= 0x40300 && VC_GCC < 0x40400
             ::Vc::Warnings::_operator_bracket_warning();
 #endif
             return d.m(index);
         }
-        Vc_ALWAYS_INLINE EntryType operator[](int index) const {
+        Vc_ALWAYS_INLINE EntryType operator[](size_t index) const {
             return d.m(index);
         }
 
@@ -454,6 +455,7 @@ static Vc_ALWAYS_INLINE double_v max(const double_v &x, const double_v &y) { ret
 #include "forceToRegisters.tcc"
 } // namespace AVX
 } // namespace Vc
+/*OUTER_NAMESPACE_END*/
 
 #include "vector.tcc"
 #include "math.h"
