@@ -34,6 +34,7 @@
 #include "mask.h"
 #include "writemaskedvector.h"
 
+/*OUTER_NAMESPACE_BEGIN*/
 namespace Vc
 {
 namespace Scalar
@@ -192,12 +193,12 @@ class Vector
         //postfix
         Vc_ALWAYS_INLINE Vector operator++(int) { return m_data++; }
 
-        Vc_ALWAYS_INLINE EntryType &operator[](int index) {
+        Vc_ALWAYS_INLINE EntryType &operator[](size_t index) {
             assert(index == 0); if(index) {}
             return m_data;
         }
 
-        Vc_ALWAYS_INLINE EntryType operator[](int index) const {
+        Vc_ALWAYS_INLINE EntryType operator[](size_t index) const {
             assert(index == 0); if(index) {}
             return m_data;
         }
@@ -468,6 +469,7 @@ template<typename T> class SwizzledVector : public Vector<T> {};
 
 } // namespace Scalar
 } // namespace Vc
+/*OUTER_NAMESPACE_END*/
 
 #include "vector.tcc"
 #include "math.h"
