@@ -22,12 +22,18 @@
 
 #include "../common/windows_fix_intrin.h"
 
+// The GCC xxxintrin.h headers do not make sure that the intrinsics have C linkage. This not really
+// a problem, unless there is another place where the exact same functions are declared. Then the
+// linkage must be the same, otherwise it won't compile. Such a case occurs on Windows, where the
+// intrin.h header (included indirectly via unistd.h) declares many SSE intrinsics again.
+extern "C" {
 // MMX
 #include <mmintrin.h>
 // SSE
 #include <xmmintrin.h>
 // SSE2
 #include <emmintrin.h>
+}
 
 #if defined(__GNUC__) && !defined(VC_IMPL_SSE2)
 #error "SSE Vector class needs at least SSE2"
@@ -38,9 +44,12 @@
 #include "macros.h"
 
 #ifdef __3dNOW__
+extern "C" {
 #include <mm3dnow.h>
+}
 #endif
 
+/*OUTER_NAMESPACE_BEGIN*/
 namespace Vc
 {
 namespace SSE
@@ -109,16 +118,22 @@ namespace SSE
             _mm_xor_si128(a, _mm_setmin_epi32()), _mm_xor_si128(b, _mm_setmin_epi32())); }
 } // namespace SSE
 } // namespace Vc
+/*OUTER_NAMESPACE_END*/
 
 // SSE3
 #ifdef VC_IMPL_SSE3
+extern "C" {
 #include <pmmintrin.h>
+}
 #elif defined _PMMINTRIN_H_INCLUDED
 #error "SSE3 was disabled but something includes <pmmintrin.h>. Please fix your code."
 #endif
 // SSSE3
 #ifdef VC_IMPL_SSSE3
+extern "C" {
 #include <tmmintrin.h>
+}
+/*OUTER_NAMESPACE_BEGIN*/
 namespace Vc
 {
 namespace SSE
@@ -136,9 +151,11 @@ namespace SSE
 
 } // namespace SSE
 } // namespace Vc
+/*OUTER_NAMESPACE_END*/
 #elif defined _TMMINTRIN_H_INCLUDED
 #error "SSSE3 was disabled but something includes <tmmintrin.h>. Please fix your code."
 #else
+/*OUTER_NAMESPACE_BEGIN*/
 namespace Vc
 {
 namespace SSE
@@ -208,16 +225,20 @@ namespace SSE
 
 } // namespace SSE
 } // namespace Vc
+/*OUTER_NAMESPACE_END*/
 
 #endif
 
 // SSE4.1
 #ifdef VC_IMPL_SSE4_1
+extern "C" {
 #include <smmintrin.h>
+}
 #else
 #ifdef _SMMINTRIN_H_INCLUDED
 #error "SSE4.1 was disabled but something includes <smmintrin.h>. Please fix your code."
 #endif
+/*OUTER_NAMESPACE_BEGIN*/
 namespace Vc
 {
 namespace SSE
@@ -422,15 +443,23 @@ namespace SSE
 
 } // namespace SSE
 } // namespace Vc
+/*OUTER_NAMESPACE_END*/
+#endif
+
+#ifdef VC_IMPL_POPCNT
+#include <popcntintrin.h>
 #endif
 
 // SSE4.2
 #ifdef VC_IMPL_SSE4_2
+extern "C" {
 #include <nmmintrin.h>
+}
 #elif defined _NMMINTRIN_H_INCLUDED
 #error "SSE4.2 was disabled but something includes <nmmintrin.h>. Please fix your code."
 #endif
 
+/*OUTER_NAMESPACE_BEGIN*/
 namespace Vc
 {
 namespace SSE
@@ -530,10 +559,13 @@ namespace SSE
     }
 } // namespace SSE
 } // namespace Vc
+/*OUTER_NAMESPACE_END*/
 
 // XOP / FMA4
 #if defined(VC_IMPL_XOP) || defined(VC_IMPL_FMA4)
+extern "C" {
 #include <x86intrin.h>
+}
 #endif
 
 #include "undomacros.h"
