@@ -23,6 +23,7 @@
 #include "intrinsics.h"
 #include "macros.h"
 
+/*OUTER_NAMESPACE_BEGIN*/
 namespace Vc
 {
 namespace SSE
@@ -93,11 +94,6 @@ template<unsigned int VectorSize> class Mask
         Vc_ALWAYS_INLINE Vc_PURE bool operator==(const Mask &rhs) const { return MaskHelper<VectorSize>::cmpeq (k, rhs.k); }
         Vc_ALWAYS_INLINE Vc_PURE bool operator!=(const Mask &rhs) const { return MaskHelper<VectorSize>::cmpneq(k, rhs.k); }
 
-        Vc_ALWAYS_INLINE Vc_PURE Mask operator&&(const Mask &rhs) const { return _mm_and_ps(k, rhs.k); }
-        Vc_ALWAYS_INLINE Vc_PURE Mask operator& (const Mask &rhs) const { return _mm_and_ps(k, rhs.k); }
-        Vc_ALWAYS_INLINE Vc_PURE Mask operator||(const Mask &rhs) const { return _mm_or_ps (k, rhs.k); }
-        Vc_ALWAYS_INLINE Vc_PURE Mask operator| (const Mask &rhs) const { return _mm_or_ps (k, rhs.k); }
-        Vc_ALWAYS_INLINE Vc_PURE Mask operator^ (const Mask &rhs) const { return _mm_xor_ps(k, rhs.k); }
         Vc_ALWAYS_INLINE Vc_PURE Mask operator!() const { return _mm_andnot_si128(dataI(), _mm_setallone_si128()); }
 
         Vc_ALWAYS_INLINE Mask &operator&=(const Mask &rhs) { k = _mm_and_ps(k, rhs.k); return *this; }
@@ -153,6 +149,9 @@ template<unsigned int VectorSize> class Mask
         Vc_ALWAYS_INLINE_L Vc_PURE_L int firstOne() const Vc_ALWAYS_INLINE_R Vc_PURE_R;
 
     private:
+#ifdef VC_COMPILE_BENCHMARKS
+    public:
+#endif
         _M128 k;
 };
 
@@ -186,8 +185,8 @@ struct ForeachHelper
 };
 
 #define Vc_foreach_bit(_it_, _mask_) \
-    for (Vc::SSE::ForeachHelper _Vc_foreach_bit_helper((_mask_).toInt()); _Vc_foreach_bit_helper.outer(); ) \
-        for (_it_ = _Vc_foreach_bit_helper.next(); _Vc_foreach_bit_helper.inner(); _Vc_foreach_bit_helper.noBreak())
+    for (Vc::SSE::ForeachHelper Vc__make_unique(foreach_bit_obj)((_mask_).toInt()); Vc__make_unique(foreach_bit_obj).outer(); ) \
+        for (_it_ = Vc__make_unique(foreach_bit_obj).next(); Vc__make_unique(foreach_bit_obj).inner(); Vc__make_unique(foreach_bit_obj).noBreak())
 
 template<unsigned int Size> Vc_ALWAYS_INLINE Vc_PURE int Mask<Size>::shiftMask() const
 {
@@ -258,7 +257,7 @@ template<> Vc_ALWAYS_INLINE Vc_PURE int Mask<2>::count() const
 
 template<> Vc_ALWAYS_INLINE Vc_PURE int Mask<4>::count() const
 {
-#ifdef VC_IMPL_SSE4_2
+#ifdef VC_IMPL_POPCNT
     return _mm_popcnt_u32(_mm_movemask_ps(data()));
 //X     tmp = (tmp & 5) + ((tmp >> 1) & 5);
 //X     return (tmp & 3) + ((tmp >> 2) & 3);
@@ -272,7 +271,7 @@ template<> Vc_ALWAYS_INLINE Vc_PURE int Mask<4>::count() const
 
 template<> Vc_ALWAYS_INLINE Vc_PURE int Mask<8>::count() const
 {
-#ifdef VC_IMPL_SSE4_2
+#ifdef VC_IMPL_POPCNT
     return _mm_popcnt_u32(_mm_movemask_epi8(dataI())) / 2;
 #else
 //X     int tmp = _mm_movemask_epi8(dataI());
@@ -290,7 +289,7 @@ template<> Vc_ALWAYS_INLINE Vc_PURE int Mask<8>::count() const
 template<> Vc_ALWAYS_INLINE Vc_PURE int Mask<16>::count() const
 {
     int tmp = _mm_movemask_epi8(dataI());
-#ifdef VC_IMPL_SSE4_2
+#ifdef VC_IMPL_POPCNT
     return _mm_popcnt_u32(tmp);
 #else
     tmp = (tmp & 0x5555) + ((tmp >> 1) & 0x5555);
@@ -344,7 +343,7 @@ class Float8Mask
         }
         Vc_ALWAYS_INLINE Vc_PURE bool operator!=(const Float8Mask &rhs) const {
             return MaskHelper<PartialSize>::cmpneq(k[0], rhs.k[0])
-                && MaskHelper<PartialSize>::cmpneq(k[1], rhs.k[1]);
+                || MaskHelper<PartialSize>::cmpneq(k[1], rhs.k[1]);
         }
 
         Vc_ALWAYS_INLINE Vc_PURE Float8Mask operator&&(const Float8Mask &rhs) const {
@@ -420,9 +419,15 @@ class Float8Mask
 #endif
         }
         Vc_ALWAYS_INLINE Vc_PURE bool isMix() const {
+            // consider [1111 0000]
+            // solution:
+            // if k[0] != k[1] => return true
+            // if k[0] == k[1] => return k[0].isMix
 #ifdef VC_USE_PTEST
-            return _mm_test_mix_ones_zeros(_mm_castps_si128(k[0]), _mm_castps_si128(k[0])) &&
-            _mm_test_mix_ones_zeros(_mm_castps_si128(k[1]), _mm_castps_si128(k[1]));
+            __m128i tmp = _mm_castps_si128(_mm_xor_ps(k[0], k[1]));
+            // tmp == 0 <=> k[0] == k[1]
+            return !_mm_testz_si128(tmp, tmp) ||
+                _mm_test_mix_ones_zeros(_mm_castps_si128(k[0]), _mm_setallone_si128());
 #else
             const int tmp = _mm_movemask_ps(k[0]) + _mm_movemask_ps(k[1]);
             return tmp > 0x0 && tmp < (0xf + 0xf);
@@ -445,7 +450,7 @@ class Float8Mask
         }
 
         Vc_ALWAYS_INLINE Vc_PURE int count() const {
-#ifdef VC_IMPL_SSE4_2
+#ifdef VC_IMPL_POPCNT
 		return _mm_popcnt_u32(toInt());
 #else
 //X             int tmp1 = _mm_movemask_ps(k[0]);
@@ -464,6 +469,9 @@ class Float8Mask
         Vc_ALWAYS_INLINE_L Vc_PURE_L int firstOne() const Vc_ALWAYS_INLINE_R Vc_PURE_R;
 
     private:
+#ifdef VC_COMPILE_BENCHMARKS
+    public:
+#endif
         M256 k;
 };
 
@@ -531,8 +539,39 @@ class Float8GatherMask
 //X             _sse_vector_foreach_it = _sse_bitscan_initialized(_sse_vector_foreach_it, mask.data()))
 //X         for (int _sse_vector_foreach_inner = 1, it = _sse_vector_foreach_it; _sse_vector_foreach_inner; --_sse_vector_foreach_inner)
 
+// Operators
+// let binary and/or/xor work for any combination of masks (as long as they have the same sizeof)
+template<unsigned int LSize, unsigned int RSize> Mask<LSize> operator& (const Mask<LSize> &lhs, const Mask<RSize> &rhs) { return _mm_and_ps(lhs.data(), rhs.data()); }
+template<unsigned int LSize, unsigned int RSize> Mask<LSize> operator| (const Mask<LSize> &lhs, const Mask<RSize> &rhs) { return _mm_or_ps (lhs.data(), rhs.data()); }
+template<unsigned int LSize, unsigned int RSize> Mask<LSize> operator^ (const Mask<LSize> &lhs, const Mask<RSize> &rhs) { return _mm_xor_ps(lhs.data(), rhs.data()); }
+
+// binary and/or/xor cannot work with one operand larger than the other
+template<unsigned int Size> void operator& (const Mask<Size> &lhs, const Float8Mask &rhs);
+template<unsigned int Size> void operator| (const Mask<Size> &lhs, const Float8Mask &rhs);
+template<unsigned int Size> void operator^ (const Mask<Size> &lhs, const Float8Mask &rhs);
+template<unsigned int Size> void operator& (const Float8Mask &rhs, const Mask<Size> &lhs);
+template<unsigned int Size> void operator| (const Float8Mask &rhs, const Mask<Size> &lhs);
+template<unsigned int Size> void operator^ (const Float8Mask &rhs, const Mask<Size> &lhs);
+
+// disable logical and/or for incompatible masks
+template<unsigned int LSize, unsigned int RSize> void operator&&(const Mask<LSize> &lhs, const Mask<RSize> &rhs);
+template<unsigned int LSize, unsigned int RSize> void operator||(const Mask<LSize> &lhs, const Mask<RSize> &rhs);
+template<unsigned int Size> void operator&&(const Mask<Size> &lhs, const Float8Mask &rhs);
+template<unsigned int Size> void operator||(const Mask<Size> &lhs, const Float8Mask &rhs);
+template<unsigned int Size> void operator&&(const Float8Mask &rhs, const Mask<Size> &lhs);
+template<unsigned int Size> void operator||(const Float8Mask &rhs, const Mask<Size> &lhs);
+
+// logical and/or for compatible masks
+template<unsigned int Size> Vc_ALWAYS_INLINE Vc_PURE Mask<Size> operator&&(const Mask<Size> &lhs, const Mask<Size> &rhs) { return _mm_and_ps(lhs.data(), rhs.data()); }
+template<unsigned int Size> Vc_ALWAYS_INLINE Vc_PURE Mask<Size> operator||(const Mask<Size> &lhs, const Mask<Size> &rhs) { return _mm_or_ps (lhs.data(), rhs.data()); }
+Vc_ALWAYS_INLINE Vc_PURE Mask<8> operator&&(const Float8Mask &rhs, const Mask<8> &lhs) { return static_cast<Mask<8> >(rhs) && lhs; }
+Vc_ALWAYS_INLINE Vc_PURE Mask<8> operator||(const Float8Mask &rhs, const Mask<8> &lhs) { return static_cast<Mask<8> >(rhs) || lhs; }
+Vc_ALWAYS_INLINE Vc_PURE Mask<8> operator&&(const Mask<8> &rhs, const Float8Mask &lhs) { return rhs && static_cast<Mask<8> >(lhs); }
+Vc_ALWAYS_INLINE Vc_PURE Mask<8> operator||(const Mask<8> &rhs, const Float8Mask &lhs) { return rhs || static_cast<Mask<8> >(lhs); }
+
 } // namespace SSE
 } // namespace Vc
+/*OUTER_NAMESPACE_END*/
 
 #include "undomacros.h"
 
