@@ -38,6 +38,7 @@
 #undef isnan
 #endif
 
+/*OUTER_NAMESPACE_BEGIN*/
 namespace Vc
 {
 namespace SSE
@@ -91,7 +92,7 @@ class WriteMaskedVector
             vec->assign(VectorHelper<T>::mul(vec->data(), x.data()), mask);
             return *vec;
         }
-        Vc_INTRINSIC Vc_CONST Vector<T> &operator/=(const Vector<T> &x);
+        Vc_INTRINSIC Vector<T> &operator/=(const Vector<T> &x);
 
         Vc_INTRINSIC Vector<T> &operator+=(EntryType x) {
             return operator+=(Vector<T>(x));
@@ -139,6 +140,9 @@ template<typename T> class Vector
 {
     friend class WriteMaskedVector<T>;
     protected:
+#ifdef VC_COMPILE_BENCHMARKS
+    public:
+#endif
         typedef typename VectorTraits<T>::StorageType StorageType;
         StorageType d;
         typedef typename VectorTraits<T>::GatherMaskType GatherMask;
@@ -536,6 +540,7 @@ Vc_ALWAYS_INLINE void forceToRegisters(const Vector<float8> &/*x1*/) {
 #endif
 } // namespace SSE
 } // namespace Vc
+/*OUTER_NAMESPACE_END*/
 
 #include "undomacros.h"
 #include "vector.tcc"
