@@ -20,6 +20,12 @@
 #ifndef UNITTEST_H
 #define UNITTEST_H
 
+#ifdef VC_ASSERT
+#error "include unittest.h before any Vc header"
+#endif
+static void unittest_assert(bool cond, const char *code, const char *file, int line);
+#define VC_ASSERT(cond) unittest_assert(cond, #cond, __FILE__, __LINE__);
+
 #include <Vc/Vc>
 #include <iostream>
 #include <iomanip>
@@ -234,6 +240,7 @@ template<typename T1, typename T2> static inline bool unittest_compareHelper( co
 template<> inline bool unittest_compareHelper<Vc::int_v, Vc::int_v>( const Vc::int_v &a, const Vc::int_v &b ) { return (a == b).isFull(); }
 template<> inline bool unittest_compareHelper<Vc::uint_v, Vc::uint_v>( const Vc::uint_v &a, const Vc::uint_v &b ) { return (a == b).isFull(); }
 template<> inline bool unittest_compareHelper<Vc::float_v, Vc::float_v>( const Vc::float_v &a, const Vc::float_v &b ) { return (a == b).isFull(); }
+template<> inline bool unittest_compareHelper<Vc::sfloat_v, Vc::sfloat_v>( const Vc::sfloat_v &a, const Vc::sfloat_v &b ) { return (a == b).isFull(); }
 template<> inline bool unittest_compareHelper<Vc::double_v, Vc::double_v>( const Vc::double_v &a, const Vc::double_v &b ) { return (a == b).isFull(); }
 template<> inline bool unittest_compareHelper<Vc::ushort_v, Vc::ushort_v>( const Vc::ushort_v &a, const Vc::ushort_v &b ) { return (a == b).isFull(); }
 template<> inline bool unittest_compareHelper<Vc::short_v, Vc::short_v>( const Vc::short_v &a, const Vc::short_v &b ) { return (a == b).isFull(); }
@@ -262,16 +269,16 @@ template<> inline bool unittest_fuzzyCompareHelper<float>( const float &a, const
     return ulpDiffToReferenceWrapper(a, b) <= _unit_test_global.float_fuzzyness;
 }
 template<> inline bool unittest_fuzzyCompareHelper<Vc::float_v>( const Vc::float_v &a, const Vc::float_v &b ) {
-    return ulpDiffToReferenceWrapper(a, b) <= _unit_test_global.float_fuzzyness;
+    return (ulpDiffToReferenceWrapper(a, b) <= _unit_test_global.float_fuzzyness).isFull();
 }
 template<> inline bool unittest_fuzzyCompareHelper<Vc::sfloat_v>( const Vc::sfloat_v &a, const Vc::sfloat_v &b ) {
-    return ulpDiffToReferenceWrapper(a, b) <= _unit_test_global.float_fuzzyness;
+    return (ulpDiffToReferenceWrapper(a, b) <= _unit_test_global.float_fuzzyness).isFull();
 }
 template<> inline bool unittest_fuzzyCompareHelper<double>( const double &a, const double &b ) {
     return ulpDiffToReferenceWrapper(a, b) <= _unit_test_global.double_fuzzyness;
 }
 template<> inline bool unittest_fuzzyCompareHelper<Vc::double_v>( const Vc::double_v &a, const Vc::double_v &b ) {
-    return ulpDiffToReferenceWrapper(a, b) <= _unit_test_global.double_fuzzyness;
+    return (ulpDiffToReferenceWrapper(a, b) <= _unit_test_global.double_fuzzyness).isFull();
 }
 
 template<typename T1, typename T2, typename M> inline void unitttest_comparePrintHelper(const T1 &a, const T2 &b, const M &m, const char *aa, const char *bb, const char *file, int line, double fuzzyness = 0.) {
@@ -350,11 +357,9 @@ class _UnitTest_Compare
         Vc_ALWAYS_INLINE _UnitTest_Compare(const char *_file, int _line)
             : m_ip(getIp()), m_failed(true)
         {
-            if (VC_IS_UNLIKELY(m_failed)) {
-                printFirst();
-                printPosition(_file, _line);
-                print(' ');
-            }
+            printFirst();
+            printPosition(_file, _line);
+            print(":\n");
         }
 
         template<typename T> Vc_ALWAYS_INLINE const _UnitTest_Compare &operator<<(const T &x) const {
@@ -527,8 +532,7 @@ static void unittest_assert(bool cond, const char *code, const char *file, int l
         if (_unit_test_global.expect_assert_failure) {
             ++_unit_test_global.assert_failure;
         } else {
-            std::cout << "       " << code << " at " << file << ":" << line << " failed.\n";
-            std::abort();
+            _UnitTest_Compare(file, line) << "assert(" << code << ") failed.";
         }
     }
 }
